==> immupdate-1.2.2.tgz: package/es/immupdate.js <==
//--------------------------------------
//  Shallow update
//--------------------------------------
/** Performs a shallow update of an object using a partial object of the same shape. A new object is returned. */
export function update(host, spec) {
    var result = clone(host);
    var hasChanged = false;
    for (var key in spec) {
        var specValue = spec[key];
        if (specValue === DELETE) {
            delete result[key];
            hasChanged = hasChanged || key in host;
        }
        else {
            result[key] = specValue;
            hasChanged = hasChanged || host[key] !== specValue;
        }
    }
    return hasChanged ? result : host;
}
// We lie about the public type so that only a property that is optional or that can be assigned to undefined can be DELETE'd
/** Marker used to delete a key */
export var DELETE = {};
var _Updater = /** @class */ (function () {
    function _Updater(data) {
        this.data = data;
    }
    _Updater.prototype.at = function (keyOrIndex) {
        return new _Updater({ type: 'at', parent: this, field: keyOrIndex });
    };
    _Updater.prototype.set = function (value) {
        var _this = this;
        var doSet = function (target) {
            var result = _this.cloneForUpdate(target);
            if (result.name === 'aborted')
                return target;
            var clonedTarget = result.clonedTarget, leafHost = result.leafHost, field = result.field;
            value === DELETE ? delete leafHost[field] : leafHost[field] = value;
            return clonedTarget;
        };
        var boundTarget = this.findBoundTarget();
        return boundTarget
            ? doSet(boundTarget)
            : doSet;
    };
    _Updater.prototype.modify = function (modifier) {
        var _this = this;
        var doModify = function (target) {
            var result = _this.cloneForUpdate(target);
            if (result.name === 'aborted')
                return target;
            var clonedTarget = result.clonedTarget, leafHost = result.leafHost, field = result.field;
            var value = modifier(leafHost[field]);
            value === DELETE ? delete leafHost[field] : leafHost[field] = value;
            return clonedTarget;
        };
        var boundTarget = this.findBoundTarget();
        return boundTarget
            ? doModify(boundTarget)
            : doModify;
    };
    _Updater.prototype.withDefault = function (value) {
        return new _Updater({ type: 'withDefault', parent: this, defaultValue: value });
    };
    _Updater.prototype.abortIfNot = function (predicate) {
        return new _Updater({ type: 'abortIfNot', parent: this, predicate: predicate });
    };
    _Updater.prototype.abortIfUndef = function () {
        return this.abortIfNot(function (value) { return value !== undefined; });
    };
    _Updater.prototype.findBoundTarget = function () {
        var current = this;
        while (true) {
            if (current.data.type === 'root')
                return current.data.boundTarget;
            current = current.data.parent;
        }
    };
    _Updater.prototype.parentUpdaters = function () {
        var updaters = [this];
        var parentUpdater = this.data.parent;
        // Ignore the root updater
        while (parentUpdater && parentUpdater.data.parent) {
            updaters.unshift(parentUpdater);
            parentUpdater = parentUpdater.data.parent;
        }
        return updaters;
    };
    _Updater.prototype.getNextValue = function (previousHost, host, field, isLast) {
        if (this.data.type === 'at') {
            var newField = this.data.field;
            var value = host[newField];
            var nextValue = isObjectOrArray(value) ? clone(value) : value;
            var newHost_1 = isLast ? host : nextValue;
            host[this.data.field] = nextValue;
            return { host: newHost_1, field: newField };
        }
        if (this.data.type === 'abortIfNot' && this.data.predicate(host) === false) {
            return { host: host, field: field, aborted: true };
        }
        if (this.data.type === 'withDefault' && previousHost[field] === undefined) {
            var nextValue = this.data.defaultValue;
            var newHost_2 = isLast ? previousHost : nextValue;
            previousHost[field] = nextValue;
            return { host: newHost_2, field: field };
        }
        var newHost = isLast ? previousHost : host;
        return { host: newHost, field: field };
    };
    _Updater.prototype.cloneForUpdate = function (target) {
        var updaters = this.parentUpdaters();
        var obj = clone(target);
        var previousHost = obj;
        var host = obj;
        var field = '';
        for (var i = 0; i < updaters.length; i++) {
            var result = updaters[i].getNextValue(previousHost, host, field, i === updaters.length - 1);
            if (result.aborted)
                return { name: 'aborted' };
            previousHost = host;
            host = result.host;
            field = result.field;
        }
        return {
            name: 'result',
            clonedTarget: obj,
            leafHost: host,
            field: field
        };
    };
    return _Updater;
}());
function isObjectOrArray(obj) {
    return obj !== null && typeof obj === 'object';
}
function clone(obj) {
    if (Array.isArray(obj))
        return obj.slice();
    var cloned = {};
    Object.keys(obj).forEach(function (key) { cloned[key] = obj[key]; });
    return cloned;
}
export function deepUpdate(target) {
    return new _Updater({ type: 'root', boundTarget: target });
}

==> immupdate-1.2.3.tgz: package/es/immupdate.js <==
//--------------------------------------
//  Shallow update
//--------------------------------------
/** Performs a shallow update of an object using a partial object of the same shape. A new object is returned. */
export function update(host, spec) {
    var result = clone(host);
    var hasChanged = false;
    for (var key in spec) {
        var specValue = spec[key];
        if (specValue === DELETE) {
            delete result[key];
            hasChanged = hasChanged || key in host;
        }
        else {
            result[key] = specValue;
            hasChanged = hasChanged || host[key] !== specValue;
        }
    }
    return hasChanged ? result : host;
}
// We lie about the public type so that only a property that is optional or that can be assigned to undefined can be DELETE'd
/** Marker used to delete a key */
export var DELETE = {};
var _Updater = /** @class */ (function () {
    function _Updater(data) {
        this.data = data;
    }
    _Updater.prototype.at = function (keyOrIndex) {
        return new _Updater({ type: 'at', parent: this, field: keyOrIndex });
    };
    _Updater.prototype.set = function (value) {
        return this.modify(function (_) { return value; });
    };
    _Updater.prototype.modify = function (modifier) {
        var _this = this;
        var doModify = function (target) {
            var result = _this.cloneForUpdate(target);
            if (result.name === 'aborted')
                return target;
            var clonedTarget = result.clonedTarget, leafHost = result.leafHost, field = result.field, structurallyModified = result.structurallyModified;
            var currentValue = leafHost[field];
            var value = modifier(currentValue);
            var changed = structurallyModified;
            if (value === DELETE) {
                if (field in leafHost)
                    changed = true;
                delete leafHost[field];
            }
            else {
                if (currentValue !== value)
                    changed = true;
                leafHost[field] = value;
            }
            return changed ? clonedTarget : target;
        };
        var boundTarget = this.findBoundTarget();
        return boundTarget
            ? doModify(boundTarget)
            : doModify;
    };
    _Updater.prototype.withDefault = function (value) {
        return new _Updater({ type: 'withDefault', parent: this, defaultValue: value });
    };
    _Updater.prototype.abortIfNot = function (predicate) {
        return new _Updater({ type: 'abortIfNot', parent: this, predicate: predicate });
    };
    _Updater.prototype.abortIfUndef = function () {
        return this.abortIfNot(function (value) { return value !== undefined; });
    };
    _Updater.prototype.findBoundTarget = function () {
        var current = this;
        while (true) {
            if (current.data.type === 'root')
                return current.data.boundTarget;
            current = current.data.parent;
        }
    };
    _Updater.prototype.parentUpdaters = function () {
        var updaters = [this];
        var parentUpdater = this.data.parent;
        // Ignore the root updater
        while (parentUpdater && parentUpdater.data.parent) {
            updaters.unshift(parentUpdater);
            parentUpdater = parentUpdater.data.parent;
        }
        return updaters;
    };
    _Updater.prototype.getNextValue = function (previousHost, host, field, isLast) {
        if (this.data.type === 'at') {
            var newField = this.data.field;
            var value = host[newField];
            var nextValue = isObjectOrArray(value) ? clone(value) : value;
            var newHost_1 = isLast ? host : nextValue;
            host[this.data.field] = nextValue;
            return { host: newHost_1, field: newField };
        }
        if (this.data.type === 'abortIfNot' && this.data.predicate(host) === false) {
            return { host: host, field: field, aborted: true };
        }
        if (this.data.type === 'withDefault' && previousHost[field] === undefined) {
            var nextValue = this.data.defaultValue;
            var newHost_2 = isLast ? previousHost : nextValue;
            previousHost[field] = nextValue;
            return { host: newHost_2, field: field, structurallyModified: true };
        }
        var newHost = isLast ? previousHost : host;
        return { host: newHost, field: field };
    };
    _Updater.prototype.cloneForUpdate = function (target) {
        var updaters = this.parentUpdaters();
        var obj = clone(target);
        var previousHost = obj;
        var host = obj;
        var field = '';
        var structurallyModified = false;
        for (var i = 0; i < updaters.length; i++) {
            var result = updaters[i].getNextValue(previousHost, host, field, i === updaters.length - 1);
            if (result.aborted)
                return { name: 'aborted' };
            structurallyModified = structurallyModified || result.structurallyModified;
            previousHost = host;
            host = result.host;
            field = result.field;
        }
        return {
            name: 'result',
            clonedTarget: obj,
            leafHost: host,
            field: field,
            structurallyModified: structurallyModified
        };
    };
    return _Updater;
}());
function isObjectOrArray(obj) {
    return obj !== null && typeof obj === 'object';
}
function clone(obj) {
    if (Array.isArray(obj))
        return obj.slice();
    var cloned = {};
    Object.keys(obj).forEach(function (key) { cloned[key] = obj[key]; });
    return cloned;
}
export function deepUpdate(target) {
    return new _Updater({ type: 'root', boundTarget: target });
}
